/*! @file common.cp

	@brief Methods common to both GradeReportPage() and HistographPage().

	$Author: eeyore $
	$Date: 2004/10/18 20:57:40 $
	
	Copyright (c) 2004 eeyore@monsterworks.com
	This program is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public
	License as published by the Free Software Foundation version 2.
	
	This program is distributed in the hope that it wil be found useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the @ref GPL in
	the documentation or read the source of MonsterworksCopyright.h.
	
	You should have received a copy of the GNU General Public Licence
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307 USA.
	
*/
//! making things easier to read while keeping doxygen informed
#define MW_CGI_TR monsterworks::cgi::tablereporter

#include "common.h"

#include <string>
using std::string;
#include <vector>
using std::vector;
#include <fstream>
using std::fstream;
#include <sstream>
using std::ostringstream;

/*! We try to read in a line and convert it into a vector

	Entries are separated by tabs, lines can end with either \n or \r\n
	In both cases, the line ending will also be eaten

	The implementation assumes your line is shorter than 10K.

	@returns the vector of values in the line
*/
std::vector<std::string> MW_CGI_TR::LineToVector(
	std::ifstream* inDataStream)			//!< the file to read lines from
{
	vector<string> theVector;

	const int theBufferSize = 1024 * 10;
	char* theBuffer = static_cast<char*>(malloc(theBufferSize));
	
	inDataStream->getline(theBuffer, theBufferSize, '\n');

	string theString = theBuffer;
	
	//we've got a DOS line
	//need to trim the last \r from the UNIX line
	if (theString.at(theString.length()-1) == '\r')
	{
		theString.erase(theString.length()-1);
	}
	std::istringstream theStream(theString, std::ios_base::app);
	
	while (theStream)
	{
		theStream.getline(theBuffer, theBufferSize, '\t');
		theVector.insert(theVector.end(), theBuffer);
	}
	//grab the last item in the column
	theStream.getline(theBuffer, theBufferSize, '\n');
	theVector.insert(theVector.end(), theBuffer);
	
	free(theBuffer);
	
	return theVector;
}
