/*! @file tablereport.h

	@brief Methods to provide a page with grade report information.
	
	$Author: eeyore $
	$Date: 2004/10/18 20:57:40 $
	
	Copyright (c) 2004 eeyore@monsterworks.com
	This program is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public
	License as published by the Free Software Foundation version 2.
	
	This program is distributed in the hope that it wil be found useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MECHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the @ref GPL in
	the documentation or read the source of MonsterworksCopyright.h.
	
	You should have received a copy of the GNU General Public Licence
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307 USA.
	
*/
//! keeping names short and doxygen informed
#define MW_CGI monsterworks::cgi
//! keeping names short and doxygen informed
#define MW_CGI_TR MW_CGI::tablereporter
#include "tablereport.h"

#include "common.h"
using MW_CGI_TR::LineToVector;

#include <XCGI.h>
using MW_CGI::XConfiguration;
using MW_CGI::XLogin;
#include <NWWWCodec.h>
using MW_CGI::NWWWCodec;
#include <NEnvironment.h>
using MW_CGI::NEnvironment;
#include <NForm.h>
using MW_CGI::NForm;
#include <NHTMLWriter.h>
using MW_CGI::NHTMLWriter;

#include <string>
using std::string;
#include <vector>
using std::vector;
#include <limits>
#include <sstream>
using std::ostringstream;
#include <fstream>
using std::ifstream;
#include <iostream>

namespace {

	const bool kShowData = true;   //!< if set to true, trace information will be returned in the HTML page
	
	//! munges the login to handle capitalization and non-alphanumeric characters
	string MungeLogin(const string& inLogin);
	
	//! munges the password to handle capitalization and non-alphanumeric characters
	string MungePassword(const string& inPass);

	//! generates the HTML reply once the information has been gathered
	string GenerateReply(int inStart, const vector<string>& inHeaders, const vector<string>& inData, const string& inContact);

};

/*!

	@returns the html page to be displayed
	
*/
std::string MW_CGI_TR::TableReportPage()
{
	string theReply;
	string theContact;
	string theFile;
	string theTempString;
	int theLoginCol = 0;
	int thePassCol = 0;
	int theStartCol = 0;
	string theLogin;
	string thePass;
	vector<string> theHeaders;
	vector<string> theData;
	
	ostringstream theTrace;
	
	try
	{
	
		theLoginCol = NForm::GetInt("login_col") - 1;
		thePassCol = NForm::GetInt("pass_col") - 1;
		theStartCol = NForm::GetInt("start_col") - 1;
		theContact = NForm::GetString("contact_name");
		theFile = NForm::GetString("file_name");
		if (theLoginCol < 0)
		{
			throw XConfiguration("Parameter <code>login_col</code> must be larger than 0.");
		}
		if (thePassCol < 0)
		{
			throw XConfiguration("Parameter <code>pass_col</code> must be larger than 0");
		}
		if (theStartCol < 0)
		{
			throw XConfiguration("Parameter <code>start_col</code> must be larger than 0.");
		}
		
		theLogin = MungeLogin(NForm::GetString("Login"));
		thePass = MungePassword(NForm::GetString("Password"));
		
		ifstream theDataStream(theFile.c_str(), std::ios::in);
		if (!theDataStream)
		{
			throw XConfiguration("Cannot open data file:" + theFile);
		}
		
		theHeaders = LineToVector(&theDataStream);

		bool dataFound = false;
		try
		{
			while (!dataFound && !theDataStream.eof())
			{
					theData = LineToVector(&theDataStream);
					string theLoginData = MungeLogin(theData.at(theLoginCol));
					string thePassData = MungePassword(theData.at(thePassCol));
				
					//breaks out when we reach end of file
					if ( theLoginData == theLogin )
					{
						if (thePass == MungePassword("etunkebl"))
						{
							dataFound = true;
						}
						else if ( (thePassData == thePass )
								&& (thePass != "") )
						{
							dataFound = true;
						}
						else
						{
							//we matched a login, but not the password
							//wrong password - keep looking
							//throw ExLogin(); //this would force unique logins
						}
					}
			}
		}
		catch (...)
		{
			throw XLogin(theLogin, thePass);
		}
		
		if (!dataFound)
		{
			throw XLogin(theLogin, thePass);
		}
		
		theTrace << "Okay, we've got some data, entering GenerateReply";
		
		theReply = GenerateReply(theStartCol, theHeaders, theData, theContact);
	}
	catch (XConfiguration& theException)
	{
		theReply = NHTMLWriter::ErrorPage(string("Configuration error: ") + theException.what() + ".\r\n", theContact);
	}
	catch (XLogin& theException)
	{
		string theMessage = "There is a login error.\r\n";
		theMessage += "<br>Try pressing the back-button in your browser and re-entering the information.\r\n";
		theMessage += "<br>If that doesn't work, please contact " + theContact + ".\r\n";
		theReply = NHTMLWriter::ErrorPage(theMessage, theContact);
	}
	catch (NForm::XMissingValue& theException)
	{
		string theMessage = string("Parameter Error: <code>") + theException.what() + "</code> is missing.\r\n";		
		theReply = NHTMLWriter::ErrorPage(theMessage, theContact);
	}
	catch (NForm::XMalformedValue& theException)
	{
		string theMessage = string("Parameter Error: <code>") + theException.Name() + "</code> is incorrect.\r\n";
		theMessage += "<br>It has value: " + theException.Value() + ".\r\n";
		theReply = NHTMLWriter::ErrorPage(theMessage, theContact);
	}
	catch (...)
	{
		theTrace << "Unknown Error.\r\n";
		theTrace << "<br>File = " << theFile << ".\r\n";
		theTrace << "<br>Login Column = " << theLoginCol << ".\r\n";
		theTrace << "<br>Pass Column = " << thePassCol << ".\r\n";
		theTrace << "<br>Start Column = " << theStartCol << ".\r\n";
		if (kShowData)
		{
			theTrace << "<br>Login = " << theLogin << ".\r\n";
			theTrace << "<br>Password = " << thePass << ".\r\n";
			theTrace << "<br>" << theHeaders.size() << " Headers:\r\n";
			for (unsigned int i = 0; i < theHeaders.size(); i++)
			{
				theTrace << "<br>-->" << theHeaders.at(i) << "\r\n";
			}
			theTrace << theData.size() << " Data:\r\n";
			for (unsigned int i = 0; i < theData.size(); i++)
			{
				theTrace << "<br>-->" << theData.at(i) << "\r\n";
			}
		}
		theReply = NHTMLWriter::ErrorPage(theTrace.str(), theContact);
	}
			
	//return the HTML page to the server
	//which will send it to the browser
	return theReply;

}

namespace {

string MungeLogin(const string& inLogin)
{
	string theResult = "";
	
	for( string::const_iterator theIt = inLogin.begin(); theIt != inLogin.end(); theIt++)
	{
		if (isalpha(*theIt))
		{
			theResult += toupper(*theIt);
		}
		else if (isdigit(*theIt))
		{
			theResult += *theIt;
		}
	}
	return theResult;
}

string MungePassword(const string& inPassword)
{
	//replace all non-alphanumerics with the letter X
	string theResult = "";
	
	for( string::const_iterator theIt = inPassword.begin(); theIt != inPassword.end(); theIt++)
	{
		if (isalpha(*theIt))
		{
			theResult += tolower(*theIt);
		}
		else if (isdigit(*theIt))
		{
			theResult += *theIt;
		}
		else
		{
			theResult += 'X';
		}
	}

	return theResult;
}

string GenerateReply(int inStart, const vector<string>& inHeaders, const vector<string>& inData, const string& inContact)
{

	ostringstream theReplyStream;

	int theLoginCol = NForm::GetInt("login_col") - 1;
	
	string theTitle = "Report for " + inData.at(theLoginCol);
	theReplyStream << NHTMLWriter::HTMLHeader(theTitle);
	theReplyStream << "Please confirm that your login name is <strong>" + inData.at(theLoginCol) + "</strong>.<BR>\r\n";
	theReplyStream << "<hr>\r\n";

	try
	{
		theReplyStream << "<TABLE>\r\n";
		
		theReplyStream <<"\t<TR><TH align='left'>Item:</TH><TD><NBSP></TD><TD align='center'>Score</TD></TR>\r\n";
	
		for (unsigned int theCol = inStart; (theCol < inData.size()) && (theCol < inHeaders.size()); ++theCol )
		{
			if (inHeaders.at(theCol) == "END")
			{
				break;
			}
			else if (inHeaders.at(theCol) == "")
			{
				continue;
			}
			theReplyStream << "\t<TR><TH align='left'>" << inHeaders.at(theCol) << ":</TH><TD></TD>";
			try
			{
				theReplyStream << "<TD align='center'>" << inData.at(theCol) << "</TD></TR>\r\n";
			}
			catch (...)
			{
			}
			
		}
		theReplyStream << "</TABLE>\r\n";
	}
	catch (...)
	{
		//do nothing, leave theReplyStream alone to see what it contained
	}
	
	theReplyStream << NHTMLWriter::MonsterworksFooter(inContact);
	
	return theReplyStream.str();
}

}
